/*
 *	The Kibitz - Debugging Customization
 *
 *	(c) 2005-2012 Tomas Valla <tom@ucw.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU Lesser General Public License. See COPYING for details.
 */

#undef LOCAL_DEBUG

#include <ucw/lib.h>
#include <ucw/fastbuf.h>
#include <ucw/mempool.h>
#include <custom/custom.h>
#include <kibitz/internal.h>
#include <stdio.h>

struct phrase *custom_get_phrase(struct fastbuf *fb, struct mempool *mp)
{
	byte buf[HARD_MAX_PHRASE_LEN+1];
	byte *b;
	int a;
	double w;

	if (!(b = bgets(fb, buf, sizeof(buf)))) {
		DBG("EOF at phrase input");
		return NULL;
	}
	if (sscanf(buf, "%lf\t%d\t", &w, &a) != 2)
		die("Invalid line format!");

	b = buf;
	do b++; while (*b != '\t');
	do b++; while (*b != '\t');
	b++;
	uns len = strlen(b);

	struct phrase *p = mp_alloc(mp, sizeof(*p) + len);
	strcpy(p->text, b);
	p->text_len = len;
	p->relevance = w;
	p->attrs.results = a;

	DBG("Scanned '%s', len=%d,%d parsed phrase='%s' rel=%d results=%d", buf, len, strlen(p->text), p->text, p->relevance, a);
	return p;
}

void custom_get_words(struct phrase *p)
{
	byte *b = p->text;
	uns w = 0;

	for (;;) {
		while (*b == ' ')
			b++;

		if (!*b)
			break;
		byte *bs = b;

		while (*b && *b != ' ')
			b++;

		wordref_add(bs - p->text, b-bs);
		w++;
	}
	if (w > 1)
		wordref_add(0, p->text_len);
}

void dump_custom_struct_phrase_attrs(struct fastbuf *fb, struct phrase_attrs *at)
{
	bprintf(fb, "result_cnt=%d", at->results);
}
