/*
 * The Kibitz commandline searcher
 *
 * (c) 2005-2012 Tomas Valla <tom@ucw.cz>
 *
 * This software may be freely distributed and used according to the terms
 * of the GNU Lesser General Public License. See COPYING for details.
 */

#undef LOCAL_DEBUG

#include <kibitz/lib.h>
#include <ucw/conf.h>
#include <ucw/getopt.h>
#include <kibitz/kibitz.h>
#include <kibitz/internal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static char *options = CF_SHORT_OPTS "c:i:v";

static void NONRET usage(char *msg)
{
	if (msg)
		fputs(msg, stderr);
	fprintf(stderr, "\nUsage: ksearch [-v] [-c cfgname] [-i idxname] <queries\n"
		"  -v		make ksearch verbose\n"
		"  -c cfgname	load config file cfgname, default is etc/kibitz\n"
		"  -i idxname	load index idxname, default is index.idx\n");
	exit(1);
}

static void make_query(char *w, uns wlen)
{
	char *ow = w;
	struct kibitz_query *q = kibitz_new_query();
	kibitz_init_query(q);
#ifdef LOCAL_DEBUG
	kibitz_dump_query(q);
#endif

	uns c = 0;
	for (;;) {
		while (*w == ' ')
			w++;
		if (!*w)
			break;
		char *ws = w;
		while (*w && *w != ' ')
			w++;
		kibitz_add_expr(q, ws, w-ws, config.single_word_bonus);
		c++;
	}
	if (!*ow)
		kibitz_add_expr(q, w, wlen, config.single_word_bonus);
	if (c>1)
		kibitz_add_expr(q, ow, wlen, config.phrase_bonus);
#ifdef LOCAL_DEBUG
	kibitz_dump_query(q);
#endif
	q->max_results = 15;
	kibitz_get_results(q);
#ifdef LOCAL_DEBUG
	kibitz_dump_query(q);
#endif

	printf("Search results for '%s': %d results found\n", ow, q->num_results);
	struct kibitz_result *r;
	uns i = 0;
	CLIST_WALK(r, q->results)
		printf("  %02d: q=%d text='%s'\n", i++, r->quality, r->phrase.text);

	kibitz_free_query(q);
}

int main(int argc, char **argv)
{
	char idx[256] = "index.idx";
	char cfg[256] = "etc/kibitz";
	int opt;
	int verbose = 0;

	log_init(argv[0]);
	idx[sizeof(idx)-1] = cfg[sizeof(cfg)-1] = 0;

	while ((opt = cf_getopt(argc, argv, options, CF_NO_LONG_OPTS, NULL)) >= 0)
		switch (opt) {
		case 'c':
			strncpy(cfg, optarg, sizeof(cfg)-1);
			break;
		case 'i':
			strncpy(idx, optarg, sizeof(idx)-1);
			break;
		case 'v':
			verbose = 1;
			break;
		default:
			usage("Invalid option specified");
		}

	kibitz_init(cfg);
	if (verbose)
		cf_set("Kibitz.Verbose=1");

	kibitz_open_index(idx);

	char line[1024];
	while (fgets(line, sizeof(line), stdin)) {
		char *l = line;
		while (*l && *l != '\n')
			l++;
		*l = 0;
		make_query(line, l-line);
	}

	kibitz_close_index();
	kibitz_cleanup();
	return 0;
}
